@@ -26,8 +26,6 @@
 //   https://github.com/tcbrindle/span/blob/master/include/tcb/span.hpp
 
 
-OIIO_NAMESPACE_BEGIN
-
 // Our pre-3.0 implementation had span::size() as a signed value, because we
 // wrote it at a time that the draft of std::span said it should be signed.
 // The final C++20 std::span ended up with an unsigned size, like all the
@@ -38,10 +36,13 @@ OIIO_NAMESPACE_BEGIN
 #    define OIIO_SPAN_SIZE_IS_UNSIGNED
 #endif
 
-using span_size_t = size_t;
-using oiio_span_size_type = OIIO::span_size_t;  // back-compat alias
+OIIO_NAMESPACE_3_1_BEGIN
 
-OIIO_INLINE_CONSTEXPR span_size_t dynamic_extent = -1;
+using span_size_t = size_t;
+using oiio_span_size_type = span_size_t;  // back-compat alias
+
+inline constexpr span_size_t dynamic_extent
+    = std::numeric_limits<span_size_t>::max();
 
 
 
@@ -95,6 +96,7 @@ public:
     /// Copy constructor (copies the span pointer and length, NOT the data).
     constexpr span (const span &copy) noexcept = default;
 
+#ifndef OIIO_DOXYGEN  /* this declaration confuses doxygen */
     /// Copy constructor from a different extent (copies the span pointer and
     /// length, NOT the data). This allows for construction of `span<const T>`
     /// from `span<T>`, and for converting fixed extent to dynamic extent.
@@ -104,6 +106,7 @@ public:
                              && (extent == dynamic_extent || extent == N))>
     constexpr span (const span<U,N> &copy) noexcept
         : m_data(copy.data()), m_size(copy.size()) { }
+#endif
 
     /// Construct from T* and length.
     constexpr span (pointer data, size_type size) noexcept
@@ -118,7 +121,7 @@ public:
 
     /// Construct from a fixed-length C array.  Template magic automatically
     /// finds the length from the declared type of the array.
-    template<size_t N>
+    template<size_t N, OIIO_ENABLE_IF(Extent == dynamic_extent || Extent == N)>
     constexpr span (T (&data)[N]) noexcept : m_data(data), m_size(N) { }
 
     /// Construct from std::vector<T>.
@@ -260,6 +263,30 @@ public:
         return const_reverse_iterator(m_data - 1);
     }
 
+    /// Compare all elements of two spans for equality
+    template <class U, span_size_t Y>
+    constexpr bool operator==(span<U,Y> r) {
+#if OIIO_CPLUSPLUS_VERSION >= 20
+        return std::equal (begin(), end(), r.begin(), r.end());
+#else
+        auto lsize = size();
+        bool same = (lsize == r.size());
+        if (lsize != r.size())
+            return false;
+        for (span_size_t i = 0; i < lsize; ++i)
+            same &= (m_data[i] == r.m_data[i]);
+        // Note: If they're not the same size, the body of the loop won't run,
+        // so there can't be a buffer overrun here.
+        return same;
+#endif
+    }
+
+    /// Compare all elements of two spans for inequality
+    template <class U, span_size_t Y>
+    constexpr bool operator!= (span<U,Y> r) {
+        return !((*this) == r);
+    }
+
 private:
     pointer     m_data = nullptr;
     size_type   m_size = 0;
@@ -273,27 +300,6 @@ using cspan = span<const T, Extent>;
 
 
 
-/// Compare all elements of two spans for equality
-template <class T, span_size_t X, class U, span_size_t Y>
-constexpr bool operator== (span<T,X> l, span<U,Y> r) {
-#if OIIO_CPLUSPLUS_VERSION >= 20
-    return std::equal (l.begin(), l.end(), r.begin(), r.end());
-#else
-    auto lsize = l.size();
-    bool same = (lsize == r.size());
-    for (span_size_t i = 0; same && i < lsize; ++i)
-        same &= (l[i] == r[i]);
-    return same;
-#endif
-}
-
-/// Compare all elements of two spans for inequality
-template <class T, span_size_t X, class U, span_size_t Y>
-constexpr bool operator!= (span<T,X> l, span<U,Y> r) {
-    return !(l == r);
-}
-
-
 
 /// span_strided<T> : a non-owning, mutable reference to a contiguous
 /// array with known length and optionally non-default strides through the
@@ -382,6 +388,26 @@ public:
     constexpr reference back() const noexcept { return (*this)[size()-1]; }
     constexpr pointer data() const noexcept { return m_data; }
 
+    /// Compare all elements of two spans for equality
+    template <class U, span_size_t Y>
+    constexpr bool operator== (span_strided<U,Y> r) {
+        auto lsize = size();
+        if (lsize != r.size())
+            return false;
+        for (span_size_t i = 0; i < lsize; ++i)
+            if ((*this)[i] != r[i])
+                return false;
+        // Note: If they're not the same size, the body of the loop won't run,
+        // so there can't be a buffer overrun here.
+        return true;
+    }
+
+    /// Compare all elements of two spans for inequality
+    template <class U, span_size_t Y>
+    constexpr bool operator!= (span_strided<U,Y> r) {
+        return !((*this) == r);
+    }
+
 private:
     pointer       m_data   = nullptr;
     size_type     m_size   = 0;
@@ -396,25 +422,6 @@ using cspan_strided = span_strided<const T, Extent>;
 
 
 
-/// Compare all elements of two spans for equality
-template <class T, span_size_t X, class U, span_size_t Y>
-constexpr bool operator== (span_strided<T,X> l, span_strided<U,Y> r) {
-    auto lsize = l.size();
-    if (lsize != r.size())
-        return false;
-    for (span_size_t i = 0; i < lsize; ++i)
-        if (l[i] != r[i])
-            return false;
-    return true;
-}
-
-/// Compare all elements of two spans for inequality
-template <class T, span_size_t X, class U, span_size_t Y>
-constexpr bool operator!= (span_strided<T,X> l, span_strided<U,Y> r) {
-    return !(l == r);
-}
-
-
 // clang-format on
 
 
@@ -586,6 +593,18 @@ as_bytes_ref(const T& ref) noexcept
 
 
 
+/// Copy the memory contents of `src` to `dst`. They must have the same
+/// total size.
+template<typename T, typename S>
+inline void
+spancpy(span<T> dst, span<S> src)
+{
+    OIIO_DASSERT(dst.size_bytes() == src.size_bytes());
+    memcpy(dst.data(), src.data(), src.size_bytes());
+}
+
+
+
 /// Try to copy `n` items of type `T` from `src[srcoffset...]` to
 /// `dst[dstoffset...]`. Don't read or write outside the respective span
 /// boundaries. Return the number of items actually copied, which should be
@@ -697,6 +716,8 @@ check_span(span<SpanType> s, const PtrType* ptr, size_t len = 1)
 }
 
 
+OIIO_NAMESPACE_3_1_END
+
 
 /// OIIO_ALLOCASPAN is used to allocate smallish amount of memory on the
 /// stack, equivalent of C99 type var_name[size], and then return a span
@@ -704,9 +725,6 @@ check_span(span<SpanType> s, const PtrType* ptr, size_t len = 1)
 #define OIIO_ALLOCA_SPAN(type, size) span<type>(OIIO_ALLOCA(type, size), size)
 
 
-OIIO_NAMESPACE_END
-
-
 
 // Declare std::size and std::ssize for our span.
 namespace std {
